/*
    FEARLUS model0-5-1-5: ClassInfo.m
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


This is the implementatin of the ClassInfo class.

*/

#import "ClassInfo.h"
#import <collections.h>


static ClassInfo *info;

@implementation ClassInfo

+(void)initialise: (id <Zone>)aZone {
  void *enum_state;
  struct list_of_classes {
    Class theClass;
    struct list_of_classes *next;
  } *l, *m, *n;
  Class aClass;
  struct objc_protocol_list *pl;

  info = [super create: aZone];

  /* Loop through all the classes. For some reason this is buggered up if you
     make any method calls during the loop. So, first we build the list using
     conventional memory. Then we'll put it into a swarm list. */
  n = l = NULL;
  while((aClass = objc_next_class(&enum_state)) != Nil) {
    m = n;
    n = (struct list_of_classes *)xmalloc(sizeof(struct list_of_classes));
    if(l == NULL) l = n;
    if(n == NULL) {
      fprintf(stderr, "Memory allocation error\n");
      exit(1);
    }
    n->theClass = aClass;
    n->next = NULL;
    if(m != NULL) m->next = n;
  }

  /* Right, now we build the swarm list. The Class type is a pointer, so it's
     OK to treat it like an id. Whilst we build the Swarm list, we destroy
     the one we've just created. Thus all memory for the object sits in the
     Swarm Zone, and the object can be destroyed when the Zone is destroyed. */

  info->cl = [List create: aZone];

  m = NULL;
  for(n = l; n != NULL; n = n->next) {
    if(m != NULL) free(m);
    /* There's a bug that conformsTo: does not work for classes without
       parents. The following test filters out those classes. */
    if(!n->theClass->protocols) continue;
    if(n->theClass != Nil) [info->cl addFirst: n->theClass];
    m = n;
  }
  if(m != NULL) free(m);
}

+(void)getClassesForProtocol: (Protocol *)p inList: (id <List>)l {
  id <Index> i = [info->cl begin: scratchZone];
  Class c;

  for(c = (Class)[i next]; [i getLoc] == Member; c = (Class)[i next]) {
    if([c conformsTo: p]) [l addFirst: c];
  }

  [i drop];

}

@end
