/*
    FEARLUS model0-5-1-5: LandAllocator.h
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Interface for the LandAllocator class. This class controls the land managers
-- killing ones that lose all their land parcels, and creating new land
managers as needed. That sort of thing.

*/

#import <objectbase/SwarmObject.h>
#import <random.h>
#import <collections.h>
#import <stdio.h>

@class Environment, LandManager, Parameter, LandParcel;

@interface LandAllocator: SwarmObject {
  // Instance variables with a causal influence
  Parameter *parameter;
  id landManagers;
  id deadLandManagers;
  id justDeadLandManagers;
  id deadDeadLandManagers;
  id parcelsForSale;
  Environment *environment;
  // Instance variables for observation purposes only
  int maxColour, minColour, nextColour;
  double barnCredit;   		/* Keep track of the amount of barn lost
				   through creating new land managers (each
				   new land manager gets their first parcel
				   free, so logically the landAllocator bears
				   the cost) and killing off land managers
				   who die in debt. The overall wealth created
				   during the simulation should be the sum of
				   the barns of the current land managers minus
				   the barnCredit */
}

-(void)setMinColour: (int)min max: (int)max;
-(void)initialiseWithEnvironment: (Environment *)e;
-(void)setParameters: (Parameter *)p;
-(void)addLandParcelForSale: (LandParcel *)lp;
-(void)transferLandParcels;
-(LandManager *)createNewLandManager;
-getLandManagers;
-(int)getNextFreeColour;
-(void)killLosers;
-(void)writeHistory: (FILE *)fp;
-(void)recordHistory;
-(double)getBarnCredit;
-(double)getWealth;
-(void)drop;

@end
