/*
    FEARLUS model0-5-1-5: StrategySelector.h
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


An object to select strategies for various contexts. This object should provide
a GUI (haha) and file interface to load probabilities for each strategy type.
It will be owned by the land allocator object, who will use it to determine
the strategies used by land managers in each context as they are created. It
contains an array of strategy types, which is hard-wired initialised with all
the different strategy classes there are.

Actually, I think this is going to have to be owned either by the modelSwarm
or the parameters. Whatever happens, it needs to be displayed before the
controlPanel setStateStopped bit in the observerSwarm.

No, it's going to have to be owned by a SubPopulation class, and the
SubPopulations are going to have to be created first.

*/

#import <objectbase/SwarmObject.h>
#import <objc/objc-api.h>		// objc_get_class
#import <tkobjc/Frame.h>
#import <tkobjc/Label.h>
#import <collections.h>
#import <simtools.h>
#import <errno.h>
#import <string.h>
#import <random.h>

#define NSTRATEGIES 16

@interface StrategySelector: SwarmObject {
  id <Array> strategyTypes;
  // GUI Stuff
  BOOL createdGUI;
  id <Frame> frame;
  id headingFrame;			/* This should be id <Frame>, but
					   the compiler complains about not
					   having the packwith: method */
  id <Label> classHeading, aboveHeading, belowNonImitativeHeading,
    belowImitativeHeading, initialHeading;
  unsigned widestClass, widestAbove, widestBelowNon, widestBelow, widestInit;
}

+create: aZone;
-(Class)selectAboveStrategy;
-(Class)selectBelowNonImitativeStrategy;
-(Class)selectBelowImitativeStrategy;
-(Class)selectInitialStrategy;
-loadFromFileNamed: (const char *)filename;
-saveToFileNamed: (const char *)filename;
-(void)writeHistory: (FILE *)fp;
-(void)recreateGUIStuff;
-(void)createGUIStuff;
-(void)pack;
-(void)configureWidths;
-(void)drop;

@end
