/*
    FEARLUS model0-5-1-5: VarProbeWidget2.h
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


This is an object to contain a VarProbeEntry. There is already an
(undocumented) Swarm library class called VarProbeWidget. It doesn't quite do
what we want of it, however. So, we'll subclass from VarProbeWidget and
change the methods that don't do what we want so that they do do what we
want. More specifically, VarProbeWidget is set up for the standard probe
display map (or so it seems to me) -- labels on the left, and probe entry on
the right. We *just* want the probe entry. That's all. It will also need to be
settable to a specific height and width (which I think the VarProbeWidget 
should be able to do anyway).

To use this class, do:

myWidget = [VarProbeWidget2 createBegin: [self getZone]];
[myWidget setObject: anObject];
[myWidget setProbe: aProbe];
[myWidget setParent: aFrame];
myWidget = [myWidget createEnd];

[myWidget pack];

Well, we cannot bloody subclass from VarProbeWidget, so pants to that. Instead
we'll copy the whole class and just do it from start to finish. Yes, I am
irritated by that. (The reason is that I need to over-ride createEnd, which
will require me to call [super createEnd] -- which, of course, means calling
the very method I am trying to over-ride. duh. If I don't bother with
[super createEnd]? Seg-fault.)

*/

#import <objectbase/SwarmObject.h>
#import <objectbase/VarProbe.h>
#import <tkobjc/VarProbeEntry.h>
#import <tkobjc/Frame.h>
#import <objc/objc-api.h>		// class_get_super_class
#import <simtoolsgui.h>
#import <gui.h>

@interface VarProbeWidget2: SwarmObject {
  id myObject;
  VarProbe *myProbe;
  id <Frame> myParent;
  BOOL interactiveFlag;
  id <VarProbeEntry> myEntry;
}

+createBegin: aZone;
-setObject: obj;
-setProbe: probe;
-setParent: aFrame;
-createEnd;		// Use parent as the parent frame not myLeft/myRight
-pack;			// Just pack the VarProbeEntry, not the label
-packFillLeft: (BOOL)dunno;
-(unsigned)getWidth;
-setWidth: (unsigned)aWidth;
-setVariableValue: (const char *)windowName;
-update;
-Spawn: (const char *)windowName;
-idReceive: (const char *)windowName;
-(const char *)package: (const char *)widgetName;
-(const char *)getId: (const char *)widgetName;
-(void)drop;

@end
